/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Network security libraries.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1994-2000
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *      Shailendra N Jain <shailen.n.jain@gmail.com>.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef _SAMPLE3_H
#define _SAMPLE3_H

/*
 * MACROs
 */

#define BUFFERSIZE            80
#define DIGESTSIZE            16
#define PTEXT_MAC_BUFFER_SIZE 96
#define CIPHERSIZE            96
#define BLOCKSIZE             32

#define CIPHER_HEADER         "-----BEGIN CIPHER-----"
#define CIPHER_TRAILER        "-----END CIPHER-----"
#define AESKEY_HEADER         "-----BEGIN AESKEY CKAID-----"
#define AESKEY_TRAILER        "-----END AESKEY CKAID-----"
#define MACKEY_HEADER         "-----BEGIN MACKEY CKAID-----"
#define MACKEY_TRAILER        "-----END MACKEY CKAID-----"
#define IV_HEADER             "-----BEGIN IV-----"
#define IV_TRAILER            "-----END IV-----"
#define MAC_HEADER            "-----BEGIN MAC-----"
#define MAC_TRAILER           "-----END MAC-----"
#define PAD_HEADER            "-----BEGIN PAD-----"
#define PAD_TRAILER           "-----END PAD-----"

typedef enum {
        ENCRYPT,
        DECRYPT,
        UNKNOWN
} ComandType;

typedef enum {
   SYMKEY = 0,
   MACKEY = 1,
   IV     = 2,
   MAC    = 3,
   PAD    = 4
} HeaderType;

/*
 * GatherCKA_ID
 */
SECStatus
GatherCKA_ID(PK11SymKey  *key,
             SECItem     *buf);


/*
 * GenerateSYMKey
 */
PK11SymKey *
GenerateSYMKey(PK11SlotInfo      *slot,
               CK_MECHANISM_TYPE mechanism,
               int               keySize,
               SECItem           *keyID,
               secuPWData        *pwdata);


/*
 * MacInit
 */
SECStatus
MacInit(PK11Context *ctx);


/*
 * MacUpdate
 */
SECStatus
MacUpdate(PK11Context   *ctx,
          unsigned char *msg,
          unsigned int  msgLen);

/*
 * MacFinal
 */
SECStatus
MacFinal(PK11Context   *ctx,
         unsigned char *mac,
         unsigned int  *macLen,
         unsigned int  maxLen);

/*
 * ComputeMac
 */
SECStatus
ComputeMac(PK11Context   *ctxmac,
           unsigned char *ptext,
           unsigned int  ptextLen,
           unsigned char *mac,
           unsigned int  *macLen,
           unsigned int  maxLen);

/*
 * CryptInit
 */
PK11Context *
CryptInit(PK11SymKey        *key,
          unsigned char     *iv,
          unsigned int      ivLen,
          CK_MECHANISM_TYPE type, 
          CK_ATTRIBUTE_TYPE operation);


/*
 * WriteToHeaderFile
 */
SECStatus
WriteToHeaderFile(char         *buf,
                  unsigned int len,
                  HeaderType   type,
                  PRFileDesc   *outFile);

/*
 * Crypt
 */
SECStatus
Crypt(PK11Context   *ctx,
      unsigned char *out,
      unsigned int  *outLen,
      unsigned int  maxOut,
      unsigned char *in,
      unsigned int  inLen);

/*
 * Encrypt
 */
SECStatus
Encrypt(PK11Context   *ctx,
        unsigned char *out,
        unsigned int  *outLen,
        unsigned int  maxout,
        unsigned char *in,
        unsigned int  inLen);

/*
 * EncryptInit
 */
PK11Context *
EncryptInit(PK11SymKey        *ek,
            unsigned char     *iv,
            unsigned int      ivLen,
            CK_MECHANISM_TYPE type);


/*
 * ReadHeaderFile
 */
SECStatus
ReadFromHeaderFile(const char *fileName,
                   HeaderType type,
                   SECItem    *item);

/*
 * EncryptAndMac
 */
SECStatus
EncryptAndMac(PRFileDesc    *inFile,
              PRFileDesc    *headerFile,
              PRFileDesc    *encFile,
              PK11SymKey    *ek,
              PK11SymKey    *mk,
              unsigned char *iv,
              unsigned int  ivLen,
              PRBool        ascii);

/*
 * FindKey
 */
PK11SymKey*
FindKey(PK11SlotInfo      *slot,
        CK_MECHANISM_TYPE mechanism,
        SECItem           *keyBuf,
        secuPWData        *pwdata);


/*
 * DecryptAndVerifyMac
 */
SECStatus
DecryptAndVerifyMac(const char  *outFileName,
                    char        *encryptedFileName,
                    SECItem     *cItem, 
                    SECItem     *macItem,
                    PK11SymKey  *ek,
                    PK11SymKey  *mk, 
                    SECItem     *ivItem,
                    SECItem     *padItem);

/*
 * GetIVandCKAIDSFromHeader
 */
SECStatus
GetIVandCKAIDSFromHeader(const char *cipherFileName,
                         SECItem    *ivItem, 
                         SECItem    *aesKeyItem, 
                         SECItem    *macKeyItem);

/*
 * PerformStepB
 */
SECStatus
PerformStepB(PK11SlotInfo *slot,
             const char   *dbdir,
             const char   *outFileName,
             const char   *headerFileName,
             char         *encryptedFileName,
             secuPWData   *pwdata,
             PRBool       ascii);

/*
 * PerformStepA
 */
SECStatus
PerformStepA(PK11SlotInfo *slot,
             const char   *dbdir,
             const char   *inFileName,
             const char   *headerFileName,
             const char   *encryptedFileName,
             const char   *noise,
             secuPWData   *pwdata,
             PRBool       ascii);

#endif /* _SAMPLE3_H */

